/* ccgo: igs/show.cc
 * 
 * Copyright 2002 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This program is free software; you can redistribute it and'or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 */

#include "show.hh"
#include "../settings.hh"
#include <gtkmm/stock.h>

using namespace igs;

TalkSelect::TalkSelect() :
	talk_chatter("Chatter"),
	talk_kibitz("Kibitz"),
	talk_say("Say"),
	talk_tell("Tell")
{
	Gtk::Menu * m = Gtk::manage(new Gtk::Menu);
	m->append(talk_chatter);
	m->append(talk_kibitz);
	m->append(talk_say);
	m->append(talk_tell);
	set_menu(* m);
}

TalkType TalkSelect::get_type()
{
	TalkType t;

	switch (get_history()) {
	case 0:
		t = TALK_CHATTER;
		break;
	case 1:
		t = TALK_KIBITZ;
		break;
	case 2:
		t = TALK_SAY;
		break;
	case 3:
		t = TALK_TELL;
		break;
	default: // no possible
		t = TALK_CHATTER;
	}
	return t;
}

void TalkSelect::disable_talk(TalkType t)
{
	switch (t) {
	case TALK_CHATTER:
		talk_chatter.set_sensitive(false);
		break;
	case TALK_KIBITZ:
		talk_kibitz.set_sensitive(false);
		break;
	case TALK_SAY:
		talk_say.set_sensitive(false);
		break;
	case TALK_TELL:
		talk_tell.set_sensitive(false);
		break;
	}
}
//////////////////////////////////////////////////////////////////////////////////////////////
void Show::line_input()
{
	input_line(line_box.get_line(), talk_select.get_type());
}

void Show::on_hide()
{
	if (no_close) {
		show();
		return;
	}
	show_close(this);
	delete this;
}

void Show::on_size_allocate(GtkAllocation * a)
{
	Gtk::Widget::on_size_allocate(a);
	if (a->width != my_width || a->height != my_height) {
		my_width = a->width;
		my_height = a->height;
		go::settings.set_int("igs-show-width", my_width);
		go::settings.set_int("igs-show-height", my_height);
	}
}

bool Show::handle_move(Gtk::ScrollType)
{
	std::cerr << "handle moved = " << vpaned.get_position() << std::endl;
	return true;
}

Show::Show(unsigned z) :
	part(z, 5.5),
	close_button(Gtk::Stock::CLOSE)
{
// 	Gtk::VPaned * vp = Gtk::manage(new Gtk::VPaned);
	add(vpaned);
	Gtk::VBox * vb = Gtk::manage(new Gtk::VBox);
	vb->pack_start(header_text, Gtk::PACK_SHRINK);

	Gtk::HBox * hb = Gtk::manage(new Gtk::HBox);
	vb->pack_start(* hb);
	hb->pack_start(part.get_board_area());

	Gtk::VBox * vb2 = Gtk::manage(new Gtk::VBox);
	hb->pack_end(* vb2, Gtk::PACK_SHRINK);
	vb2->pack_start(part.get_button_area());
	close_button.signal_clicked().connect(slot(* this, & Window::hide));
	vb2->pack_end(close_button, Gtk::PACK_SHRINK);

	hb = Gtk::manage(new Gtk::HBox);
	vb->pack_end(* hb, Gtk::PACK_SHRINK);
	hb->pack_start(part.get_modifi_area(), Gtk::PACK_SHRINK);
	hb->pack_end(part.get_label_area(), Gtk::PACK_SHRINK);
	vpaned.pack1(* vb);
	Gtk::ScrolledWindow * sw = Gtk::manage(new Gtk::ScrolledWindow);
	sw->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_ALWAYS);
	sw->add(message_box);
	vb = Gtk::manage(new Gtk::VBox);
	vpaned.pack2(* vb);
	vb->pack_start(* sw);
	hb = Gtk::manage(new Gtk::HBox);
	vb->pack_end(* hb, Gtk::PACK_SHRINK);
	hb->pack_start(talk_select, Gtk::PACK_SHRINK);
	hb->pack_start(line_box);
	line_box.signal_activate().connect(slot(* this, & Show::line_input));
	vaj = sw->get_vadjustment();

	message_box.set_editable(false);
	message_box.set_cursor_visible(false);
	message_box.set_wrap_mode(Gtk::WRAP_WORD);

	my_width = go::settings.get_int("igs-show-width");
	if (my_width <= 0) my_width = 320;
	my_height = go::settings.get_int("igs-show-height");
	if (my_height <= 0) my_height = 360;
	// set_default_size(my_width, my_height);
	resize(my_width, my_height);
// 	my_vpaned_position = go::settings.get_int("igs-show-vpaned-position");
// 	if (my_vpaned_position <= 0) my_vpaned_position = my_height - 30;
	my_vpaned_position = my_height - 30;
	vpaned.set_position(my_vpaned_position);
	vpaned.signal_move_handle().connect(SigC::slot(* this, & Show::handle_move));
	show_all();
	set_no_close(false);
	part.set_no_play();
	part.set_hold_action();
}

Show::~Show()
{
}

void Show::set_no_close(bool b)
{
	no_close = b;
	close_button.set_sensitive(! no_close);
}

void Show::set_play(bool w, bool b)
{
	part.set_no_play(! (w || b));
	part.set_players(w ? 0 : this, b ? 0 : this);
	if (w || b) {
		part.held_handicap.connect(input_handicap.slot());
		part.held_put.connect(input_put.slot());
		part.held_pass.connect(input_pass.slot());
		part.held_undo.connect(input_undo.slot());
		part.held_terri.connect(input_terri.slot());
		part.held_reset.connect(input_reset.slot());
		part.held_done.connect(input_done.slot());
		talk_select.set_history(TALK_SAY);
		if (w && b) talk_select.disable_talk(TALK_TELL);
	} else { // no opponent, not playing
		talk_select.disable_talk(TALK_SAY);
	}
}

void Show::set_komi(double k)
{
	part.set_komi(k);
}

int Show::move_num()
{
	return part.get_game()->get_move_num();
}

bool Show::add_move(go::Move * m)
{
	return part.add_move(m);
}

bool Show::undo_move()
{
	return part.undo_move();
}

void Show::hang_up(const std::string & s)
{
	add_message(s);
	part.stop_game();
	line_box.set_sensitive(false);
	set_no_close(false);
}

void Show::start_time(int t, int b, int m)
{
	part.start_time(t, b, m);
}

void Show::sync_time(int wt, int wm, int bt, int bm)
{
	part.time_update(wt, wm, bt, bm);
}

void Show::set_header(const std::string & s)
{
	header_text.set_text(s);
}

void Show::add_message(const std::string & msg)
{
	if (! msg.length()) return;
	bool bot = (vaj->get_value()  + vaj->get_page_size() >= vaj->get_upper());
	message_box.get_buffer()->insert(message_box.get_buffer()->end(), msg);
	message_box.get_buffer()->insert(message_box.get_buffer()->end(), "\n");
	if (bot) {
		message_box.scroll_to_mark(message_box.get_buffer()->create_mark("end", message_box.get_buffer()->end()), 0);
	}
}

void Show::show_score()
{
	part.enter_score();
}

void Show::show_restore()
{
	part.restore_board();
}

void Show::show_terri(const go::Loc & l)
{
	part.terri_request(l);
}

void Show::show_adjourn()
{
	hang_up("game adjourned");
}

void Show::show_score_board(const Board & b)
{
	go::Terri * t = new go::Terri[b.board_size * b.board_size];
	bool is_score = false;
	for (int i = 0; i < b.board_size * b.board_size; i ++) switch (b.state[i]) {
	case 2: // not a score board!
	case 6:
		is_score = false;
		goto out;
	case 4:
		t[i] = go::TERRI_WHITE;
		is_score = true;
		break;
	case 5:
		t[i] = go::TERRI_BLACK;
		is_score = true;
		break;
	default:
		t[i] = go::TERRI_NEUTRAL;
	}
 out:
	if (is_score) part.add_score(t); // don't delete t
	else delete t;
}
